@@ -19,21 +19,26 @@
 		private $method;
 		private $uri;
 		private $parts;
-		private $params;
+		private $params = array();
 		private $ip;
+		private $raw;
 		
-		public function __construct($limitedHttpMethods) {
+		public function __construct($limitedHttpMethods, $raw = FALSE) {
 			$this->method = strtolower($_SERVER['REQUEST_METHOD']);
 			$this->uri = $this->getUri();
 			$this->ip = $_SERVER['REMOTE_ADDR'];
+			$this->raw = $raw;
 			if ($limitedHttpMethods and isset($_SERVER['HTTP_MOLLIFY_HTTP_METHOD']))
 				$this->method = strtolower($_SERVER['HTTP_MOLLIFY_HTTP_METHOD']);
 			
 			$p = stripos($this->uri, "?");
 			if ($p) $this->uri = trim(substr($this->uri, 0, $p), "/");
 			
-			$this->parts = explode("/", $this->uri);
-			$this->params = array();
+			$this->parts = strlen($this->uri) > 0 ? explode("/", $this->uri) : array();
+			$this->initData();
+		}
+		
+		private function initData() {
 			$this->data = NULL;
 			
 			switch($this->method) {
@@ -43,7 +48,7 @@
 				case self::METHOD_POST:
 				case self::METHOD_PUT:
 					$this->params = $_REQUEST;
-					if (!isset($this->params['format']) or $this->params['format'] != 'binary') {
+					if (!$this->raw and (!isset($this->params['format']) or $this->params['format'] != 'binary')) {
 						$data = file_get_contents("php://input");
 						if ($data and strlen($data) > 0)
 							$this->data = json_decode($data, TRUE);
@@ -57,9 +62,16 @@
 			}
 		}
 		
+		public function getSessionId() {
+			if ($this->hasParam("session")) return $this->param("session");
+			if (isset($_SERVER['HTTP_MOLLIFY_SESSION_ID'])) return $_SERVER['HTTP_MOLLIFY_SESSION_ID'];
+			return NULL;
+		}
+		
 		private function getUri() {
 			$uri = isset($_SERVER['REQUEST_URI']) ? $_SERVER['REQUEST_URI'] : $_SERVER['PHP_SELF'];
 			$pos = strpos($uri, "/r.php/");
+			if ($pos === FALSE) return "";
 			return trim(substr($uri, $pos + 7), "/");
 		}
 		
@@ -71,8 +83,10 @@
 			return $this->uri;
 		}
 		
-		public function path() {
-			return $this->parts;
+		public function path($index = NULL) {
+			if ($index == NULL)
+				return $this->parts;
+			return $this->parts[$index];
 		}
 
 		public function ip() {
