@@ -22,11 +22,11 @@
 		
 			"INVALID_PATH" => array(201, "Invalid path", 403), 
 			"FILE_DOES_NOT_EXIST" => array(202, "File does not exist", 403),
-			"DIR_DOES_NOT_EXIST" => array(203, "Directory does not exist", 403),
+			"DIR_DOES_NOT_EXIST" => array(203, "Folder does not exist", 403),
 			"FILE_ALREADY_EXISTS" => array(204, "File already exists", 403),
-			"DIR_ALREADY_EXISTS" => array(205, "Directory already exists", 403),
+			"DIR_ALREADY_EXISTS" => array(205, "Folder already exists", 403),
 			"NOT_A_FILE" => array(206, "Target is not a file", 403),
-			"NOT_A_DIR" => array(207, "Target is not a directory", 403),
+			"NOT_A_DIR" => array(207, "Target is not a folder", 403),
 			"DELETE_FAILED" => array(208, "Could not delete", 403),
 			"NO_UPLOAD_DATA" => array(209, "No upload data available", 403),
 			"UPLOAD_FAILED" => array(210, "File upload failed", 403),
@@ -39,36 +39,60 @@
 			"UNEXPECTED_ERROR" => array(999, "Unexpected server error", 500),
 		);
 		private $output;
+		private $listeners = array();
 		
 		function __construct($output) {
 			$this->output = $output;
 		}
 		
-		public function download($filename, $type, $stream, $size = NULL, $range = NULL) {
-			$this->output->downloadBinary($filename, $type, $stream, $size, $range);
+		public function addListener($l) {
+			$this->listeners[] = $l;
+		}
+		
+		public function download($filename, $type, $mobile, $stream, $size = NULL, $range = NULL) {
+			$this->output->downloadBinary($filename, $type, $mobile, $stream, $size, $range);
+			$this->notifyResponse();
 		}
 
 		public function send($filename, $type, $stream, $size = NULL) {
 			$this->output->sendBinary($filename, $type, $stream, $size);
+			$this->notifyResponse();
 		}
 		
 		public function html($html) {
 			$this->output->sendResponse(new Response(200, "html", $html));
+			$this->notifyResponse();
 		}
 		
 		public function success($data) {
 			$this->output->sendResponse(new Response(200, "json", $this->getSuccessResponse($data)));
+			$this->notifyResponse();
+		}
+
+		public function fail($code, $error, $details = NULL) {
+			$this->output->sendResponse(new Response(403, "json", $this->getErrorResponse(array($code, $error), $details)));
+			$this->notifyResponse();
 		}
 		
 		public function error($type, $details) {
 			$error = $this->getError($type);
 			$this->output->sendResponse(new Response($error[2], "json", $this->getErrorResponse($error, $details)));
+			$this->notifyResponse();
+		}
+		
+		public function redirect($url) {
+			$this->output->redirect($url);
 		}
 		
 		public function unknownServerError($msg) {
 			$this->error("UNEXPECTED_ERROR", $msg);
 		}
 		
+		private function notifyResponse() {
+			foreach($this->listeners as $l)
+				$l->onResponseSent();
+		}
+		
 		private function getSuccessResponse($data) {
 			if (Logging::isDebug()) {
 				Logging::logDebug("RESPONSE success ".Util::toString($data));
