@@ -58,5 +58,31 @@
 			}
 			return $r."}";
 		}
+		
+		static function convertArrayCharset($a) {
+			$result = array();
+			foreach($a as $k=>$v) {
+				if (is_array($v)) $result[$k] = self::convertArrayCharset($v);
+				else $result[$k] = self::convertCharset($v);
+			}
+			return $result;
+		}
+		
+		static function convertCharset($v, $charset = NULL, $encode = TRUE) {
+			if (!$charset or $charset === NULL) {
+				if ($encode)
+					return utf8_encode($v);
+				return utf8_decode($v);
+			}
+			$from = $encode ? $charset : 'UTF-8';
+			$to = $encode ? 'UTF-8' : $charset;
+			return iconv($from, $to, $v);
+		}
+		
+		static function replaceParams($text, $values) {
+			foreach($values as $k => $v)
+				$text = str_replace('%'.$k.'%', $v, $text);
+			return $text;
+		}
 	}
 ?>
\ No newline at end of file
