@@ -1,7 +1,7 @@
 <?php
 
 	/**
-	 * Copyright (c) 2008- Samuli Jrvel
+	 * Copyright (c) 2008- Samuli Järvelä
 	 *
 	 * All rights reserved. This program and the accompanying materials
 	 * are made available under the terms of the Eclipse Public License v1.0
@@ -19,31 +19,67 @@
 		private $allowedUploadTypes;
 		private $permissionCache = array();
 		private $folderCache = array();
-		private $detailsPlugins = array();
+		private $contextPlugins = array();
+		private $dataRequestPlugins = array();
+		private $searchers = array();
+		private $filesystems = array();
+		private $idProvider;
 		
-		public $allowFilesystems = TRUE;
+		public $allowFilesystems = FALSE;
 
 		function __construct($env) {
 			require_once("MollifyFilesystem.class.php");
 			require_once("LocalFilesystem.class.php");
 			require_once("FilesystemItem.class.php");
+			require_once("BaseSearcher.class.php");
+			require_once("FilesystemSearcher.class.php");
+			require_once("CoreFileDataProvider.class.php");
+			require_once("ItemIdProvider.class.php");
 			
 			$this->env = $env;
-			$this->allowedUploadTypes = $env->settings()->setting('allowed_file_upload_types', TRUE);
+			$this->idProvider = new ItemIdProvider($env);
 			
+			$this->allowedUploadTypes = $env->settings()->setting('allowed_file_upload_types', TRUE);
+			$this->registerSearcher(new FileSystemSearcher($this->env));
+			
+			$coreData = new CoreFileDataProvider($this->env);
+			$coreData->init($this);
+						
 			FileEvent::register($this->env->events());
 		}
 		
 		public function initialize() {}
+		
+		public function itemIdProvider() {
+			return $this->idProvider;
+		}
+		
+		public function registerFilesystem($id, $factory) {
+			Logging::logDebug("Filesystem registered: ".$id);
+			$this->filesystems[$id] = $factory;
+		}
 
-		public function registerDetailsPlugin($plugin) {
-			$this->detailsPlugins[] = $plugin;
+		public function registerItemContextPlugin($key, $plugin) {
+			$this->contextPlugins[$key] = $plugin;
+		}
+
+		public function registerDataRequestPlugin($keys, $plugin) {
+			foreach($keys as $key)
+				$this->dataRequestPlugins[$key] = $plugin;
+		}
+		
+		public function getDataRequestPlugins() {
+			return $this->dataRequestPlugins;
+		}
+		
+		public function registerSearcher($searcher) {
+			$this->searchers[] = $searcher;
 		}
 		
 		public function getRootFolders($all = FALSE) {
 			$list = array();
 			
-			foreach($this->getFolderDefs($all) as $folderDef) {				
+			foreach($this->getFolderDefs($all) as $folderDef) {
 				$root = $this->filesystem($folderDef, !$all)->root();
 				if (!$this->env->authentication()->hasReadRights($this->permission($root))) continue;
 				$list[$folderDef['id']] = $root;
@@ -53,23 +89,23 @@
 		}
 		
 		private function getFolderDefs($all = FALSE) {
-			if ($this->env->configuration()->isAuthenticationRequired() and !$all)
-				$folderDefs = $this->env->configuration()->getUserFolders($this->env->authentication()->getUserId());
+			if (!$all)
+				$folderDefs = $this->env->configuration()->getUserFolders($this->env->session()->userId(), TRUE);
 			else
 				$folderDefs = $this->env->configuration()->getFolders();
 
 			$list = array();
 			
 			foreach($folderDefs as $folderDef) {
-				$root = $this->filesystem($folderDef, !$all)->root();
-				if (!$this->env->authentication()->hasReadRights($this->permission($root))) continue;
+				if (array_key_exists($folderDef['id'], $list)) continue;
+				if (!$this->isFolderValid($folderDef, !$all)) continue;
 				
 				if (!isset($folderDef["name"]) and !isset($folderDef["default_name"])) {
-					$this->env->session()->reset();
+					$this->env->session()->end();
 					throw new ServiceException("INVALID_CONFIGURATION", "Folder definition does not have a name (".$folderDef['id'].")");
 				}
 				if (!isset($folderDef["path"])) {
-					$this->env->session()->reset();
+					$this->env->session()->end();
 					throw new ServiceException("INVALID_CONFIGURATION", "Folder definition does not have a path (".$folderDef['id'].")");
 				}
 				
@@ -79,9 +115,24 @@
 			return $list;
 		}
 		
+		private function isFolderValid($folderDef, $mustExist = TRUE) {
+			$root = $this->filesystem($folderDef, $mustExist)->root();
+			if ($mustExist and !$root->exists()) throw new ServiceException("DIR_DOES_NOT_EXIST", 'root id:'.$folderDef['id']);
+			if (!$this->allowFilesystems and !$this->env->authentication()->hasReadRights($this->permission($root))) return FALSE;
+			return TRUE;
+		}
+		
 		private function createFilesystem($folderDef) {
+			if ($folderDef == NULL) throw new ServiceException("INVALID_CONFIGURATION", "Invalid root folder definition");
+
 			$id = isset($folderDef['id']) ? $folderDef['id'] : '';
 			
+			//TODO this is hack, support real filesystem types
+			if (array_key_exists("S3FS", $this->filesystems)) {
+				$factory = $this->filesystems["S3FS"];
+				return $factory->createFilesystem($id, $folderDef, $this);
+			}
+			
 			switch ($this->filesystemType($folderDef)) {
 				case MollifyFilesystem::TYPE_LOCAL:
 					return new LocalFilesystem($id, $folderDef, $this);
@@ -104,13 +155,19 @@
 				"allowed_file_upload_types" => $this->allowedFileUploadTypes()
 			);
 			
+			$this->itemIdProvider()->loadRoots();
+			
 			$result["folders"] = array();
 			foreach($this->getRootFolders() as $id => $folder) {
+				$nameParts = explode("/", str_replace("\\", "/",$folder->name()));
+				$name = array_pop($nameParts);
+				
 				$result["folders"][] = array(
-					"id" => $folder->publicId(),
-					"name" => $folder->name(),
+					"id" => $folder->id(),
+					"name" => $name,
+					"group" => implode("/", $nameParts),
 					"parent_id" => NULL,
-					"root_id" => $folder->publicId(),
+					"root_id" => $folder->id(),
 					"path" => ""
 				);
 			}
@@ -129,30 +186,26 @@
 		}
 		
 		public function item($id, $nonexisting = FALSE) {
-			$parts = explode(":".DIRECTORY_SEPARATOR, $id);
-			if (count($parts) != 2) throw new ServiceException("INVALID_CONFIGURATION", "Invalid item id: ".$id);
+			$location = $this->itemIdProvider()->getLocation($id);
+			$parts = explode(":".DIRECTORY_SEPARATOR, $location);
+			if (count($parts) != 2) throw new ServiceException("INVALID_CONFIGURATION", "Invalid item location: ".$location);
 			
 			$filesystemId = $parts[0];
 			$path = $parts[1];
-			if (strpos($path, "..") != FALSE) throw new ServiceException("INVALID_REQUEST");
+			if (strpos($path, "../") !== FALSE or strpos($path, "..\\") !== FALSE) new ServiceException("INVALID_CONFIGURATION", "Invalid item location: ".$location);
 			
 			if (array_key_exists($filesystemId, $this->folderCache)) {
 				$folderDef = $this->folderCache[$filesystemId];
 			} else {
-				if (!$this->isFolderValid($filesystemId)) throw new ServiceException("UNAUTHORIZED");
 				$folderDef = $this->env->configuration()->getFolder($filesystemId);
+				if (!$folderDef or !$this->isFolderValid($folderDef)) throw new ServiceException("UNAUTHORIZED");
+				
 				$this->folderCache[$filesystemId] = $folderDef;
 			}
 			
 			return $this->filesystem($folderDef)->createItem($id, $path, $nonexisting);
 		}
-		
-		private function isFolderValid($id) {
-			if ($this->allowFilesystems or !$this->env->configuration()->isAuthenticationRequired()) return TRUE;
-			$folders = $this->getFolderDefs();
-			return array_key_exists($id, $folders);
-		}
-		
+				
 		public function assertFilesystem($folderDef) {
 			$this->filesystem($folderDef, TRUE);
 		}
@@ -174,7 +227,8 @@
 			//make sure folder permissions are fetched into cache
 			$this->fetchPermissions($folder);
 			$this->assertRights($folder, Authentication::RIGHTS_READ, "items");
-
+			$this->itemIdProvider()->load($folder);
+			
 			$list = array();
 			foreach($folder->items() as $i) {
 				if (!$this->isItemVisible($i)) continue;
@@ -182,6 +236,11 @@
 			}
 			return $list;
 		}
+
+		public function hierarchy($folder) {
+			$this->assertRights($folder, Authentication::RIGHTS_READ, "hierarchy");
+			return $folder->hierarchy();
+		}
 		
 		private function isItemVisible($item) {
 			if ($this->env->authentication()->isAdmin()) return TRUE;
@@ -190,25 +249,34 @@
 			return FALSE;
 		}
 
-		public function details($item) {
+		public function details($item, $data = NULL) {
 			$this->assertRights($item, Authentication::RIGHTS_READ, "details");
 			
 			$details = $item->details();
 			$details["description"] = $this->description($item);
 			$details["permission"] = $this->permission($item);
 			
-			foreach($this->detailsPlugins as $p) {
-				$l = $p->getItemDetails($item, $details);
+			foreach($this->contextPlugins as $k=>$p) {
+				$d = ($data != NULL and isset($data[$k])) ? $data[$k] : NULL;
+				$l = $p->getItemContextData($item, $details, $k, $d);
 				if (!$l) continue;
-				
-				foreach($l as $k=>$v)
-					$details[$k] = $v;
+				$details[$k] = $l;
 			}
 			return $details;
 		}
-
-		public function datetimeFormat() {
-			return "YmdHis";
+		
+		public function checkExisting($folder, $files) {
+			$existing = array();
+			
+			foreach($files as $file) {
+				$f = $folder->fileWithName($file);
+				if ($f->exists()) $existing[] = $file;
+			}
+			return $existing;
+		}
+		
+		public function env() {
+			return $this->env;
 		}
 
 		public function description($item) {
@@ -235,7 +303,7 @@
 		
 		public function fetchPermissions($folder) {
 			if ($this->env->authentication()->isAdmin()) return;
-			$permissions = $this->env->configuration()->getAllItemPermissions($folder, $this->env->authentication()->getUserId());
+			$permissions = $this->env->configuration()->getAllItemPermissions($folder, $this->env->session()->userId());
 
 			$this->permissionCacheFolders[] = $folder->id();
 			foreach($permissions as $id => $p)
@@ -251,7 +319,7 @@
 				$permission = $this->permissionCache[$item->id()];
 				Logging::logDebug("Permission cache get [".$item->id()."]=".$permission);
 			} else {
-				$permission = $this->env->configuration()->getItemPermission($item, $this->env->authentication()->getUserId());
+				$permission = $this->env->configuration()->getItemPermission($item, $this->env->session()->userId());
 				if (!$permission) return $this->env->authentication()->getDefaultPermission();
 				
 				$this->permissionCache[$item->id()] = $permission;
@@ -293,16 +361,10 @@
 		public function rename($item, $name) {
 			Logging::logDebug('rename from ['.$item->path().'] to ['.$name.']');
 			$this->assertRights($item, Authentication::RIGHTS_WRITE, "rename");
-			
 			$to = $item->rename($name);
 			
-			if ($this->env->features()->isFeatureEnabled("description_update"))
-				$this->env->configuration()->moveItemDescription($item, $to);
-				
-			if ($this->env->features()->isFeatureEnabled("permission_update"))
-				$this->env->configuration()->moveItemPermissions($item, $to);
-			
-			$this->env->events()->onEvent(FileEvent::rename($item, $name));
+			$this->env->events()->onEvent(FileEvent::rename($item, $to));
+			$this->idProvider->move($item, $to);
 		}
 
 		public function copy($item, $to) {
@@ -319,14 +381,14 @@
 		}
 		
 		public function copyItems($items, $folder) {
-			Logging::logDebug('copying '.count($items).' items to ['.$to.']');
+			Logging::logDebug('copying '.count($items).' items to ['.$folder.']');
 			$this->assertRights($items, Authentication::RIGHTS_READ, "copy");
 			
 			foreach($items as $item) {
-				if ($item->isFile)
-					$this->copy($item, $folder->fileWithName($item->name(), TRUE));
+				if ($item->isFile())
+					$this->copy($item, $folder->createFile($item->name()));
 				else
-					$this->copy($item, $folder->folderWithName($item->name(), TRUE));
+					$this->copy($item, $folder->folderWithName($item->name()));
 			}
 		}
 		
@@ -338,14 +400,9 @@
 			$this->assertRights($to, Authentication::RIGHTS_WRITE, "move");
 
 			$to = $item->move($to);
-			
-			if ($this->env->features()->isFeatureEnabled("description_update"))
-				$this->env->configuration()->moveItemDescription($item, $to);
-				
-			if ($this->env->features()->isFeatureEnabled("permission_update"))
-				$this->env->configuration()->moveItemPermissions($item, $to);
-			
-			$this->env->events()->onEvent(FileEvent::move($item, $to));			
+						
+			$this->env->events()->onEvent(FileEvent::move($item, $to));
+			$this->idProvider->move($item, $to);
 		}
 		
 		public function moveItems($items, $to) {
@@ -364,13 +421,13 @@
 			
 			$item->delete();
 			
-			if ($this->env->features()->isFeatureEnabled("description_update"))
+			if ($this->env->features()->isFeatureEnabled("descriptions"))
 				$this->env->configuration()->removeItemDescription($item);
-				
-			if ($this->env->features()->isFeatureEnabled("permission_update"))
-				$this->env->configuration()->removeItemPermissions($item);
+			
+			$this->env->configuration()->removeItemPermissions($item);
 			
 			$this->env->events()->onEvent(FileEvent::delete($item));
+			$this->idProvider->delete($item);
 		}
 		
 		public function deleteItems($items) {
@@ -389,14 +446,18 @@
 			$new = $parent->createFolder($name);
 			$this->env->events()->onEvent(FileEvent::createFolder($new));
 			
-			if ($this->env->features()->isFeatureEnabled("permission_update") and !$this->env->authentication()->isAdmin())
-				$this->env->configuration()->addItemPermission($new->id(), Authentication::PERMISSION_VALUE_READWRITE, $this->env->authentication()->getUserId());
+			if (!$this->env->authentication()->isAdmin() and !in_array("permission_inheritance", $this->env->configuration()->getSupportedFeatures()))
+				$this->env->configuration()->addItemPermission($new->id(), Authentication::PERMISSION_VALUE_READWRITE, $this->env->session()->userId());
 		}
 
-		public function download($file, $range = NULL) {
+		public function download($file, $mobile, $range = NULL) {
 			if (!$range)
 				Logging::logDebug('download ['.$file->id().']');
 			$this->assertRights($file, Authentication::RIGHTS_READ, "download");
+			if (!$file->filesystem()->isDirectDownload()) {
+				$this->env->response()->redirect($file->filesystem()->getDownloadUrl($file));
+				return;
+			}
 			
 			$name = $file->name();
 			$size = $file->size();
@@ -427,26 +488,34 @@
 			if (!$range)
 				$this->env->events()->onEvent(FileEvent::download($file));
 
-			$this->env->response()->download($name, $file->extension(), $file->read($range), $size, $range);							
+			$this->env->response()->download($name, $file->extension(), $mobile, $file->read($range), $size, $range);
 		}
 
 		public function view($file) {
 			Logging::logDebug('view ['.$file->id().']');
 			$this->assertRights($file, Authentication::RIGHTS_READ, "view");
-			$this->env->events()->onEvent(FileEvent::download($file));
+			$this->env->events()->onEvent(FileEvent::view($file));
 			$this->env->response()->send($file->name(), $file->extension(), $file->read(), $file->size());
 		}
 		
+		public function updateFileContents($item, $content) {
+			if (!$item->isFile()) throw new ServiceException("NOT_A_FILE", $item->path());
+			Logging::logDebug('update file contents ['.$item->id().']');
+			$this->assertRights($item, Authentication::RIGHTS_WRITE, "update content");
+			$this->env->events()->onEvent(FileEvent::upload($item));
+			$item->put($content);
+		}
+		
+		public function getUploadTempDir() {
+			$dir = $this->env->settings()->setting("upload_temp_dir");
+			if ($dir != NULL and strlen($dir) > 0) return $dir;
+			return sys_get_temp_dir();
+		}
+		
 		public function uploadTo($folder) {
 			$this->env->features()->assertFeature("file_upload");
 			$this->assertRights($folder, Authentication::RIGHTS_WRITE, "upload");
 			
-			if ($this->env->request()->hasParam('uploader') and $this->env->request()->param('uploader') === 'plupload') {
-				require_once("plupload.php");
-				plupload($folder);
-				return;
-			}
-
 			if (!isset($_FILES['uploader-http']) and !isset($_FILES['uploader-flash']))
 				throw new ServiceException("NO_UPLOAD_DATA");
 			
@@ -454,6 +523,8 @@
 			
 			// flash uploader (uploads one file at a time)
 			if (isset($_FILES['uploader-flash'])) {
+				if (!isset($_FILES['uploader-flash']['tmp_name'])) throw new ServiceException("UPLOAD_FAILED");
+				
 				$this->upload($folder, $_FILES['uploader-flash']['name'], $_FILES['uploader-flash']['tmp_name']);
 				return;
 			}
@@ -471,30 +542,66 @@
 		
 		private function upload($folder, $name, $origin) {
 			$target = $folder->createFile($name);
+			if ($target->exists()) throw new ServiceException("FILE_ALREADY_EXISTS");
 			Logging::logDebug('uploading to ['.$target.']');
 			
-			$src = @fopen($origin, "r");
+			$src = @fopen($origin, "rb");
 			if (!$src)
 				throw new ServiceException("SAVING_FAILED", "Failed to read uploaded data");			
-			$dst = $target->write();
-			
-			while (!feof($src))
-				fwrite($dst, fread($src, 4096));
-
-			fclose($dst);
+			$target->write($src);
 			fclose($src);
 			unlink($origin);
 			
 			$this->env->events()->onEvent(FileEvent::upload($target));
 		}
 		
-		public function downloadAsZip($items) {
+		public function uploadFrom($folder, $name, $stream, $src = '[Unknown]') {
+			$this->env->features()->assertFeature("file_upload");
+			$this->assertRights($folder, Authentication::RIGHTS_WRITE, "upload");
+
+			$targetItem = $folder->createFile($name);
+			if (Logging::isDebug()) Logging::logDebug("Upload from $src ($name) to ".$targetItem->id());
+			$targetItem->write($stream);
+			
+			$this->env->events()->onEvent(FileEvent::upload($targetItem));
+		}
+		
+		public function downloadAsZip($items, $mobile) {
+			$zip = $this->createZip($items);
+			$name = "items.zip";
+			if (!is_array($items)) $name = $items->name().".zip";
+			$this->env->response()->download($name, "zip", $mobile, $zip->stream());	
+		}
+		
+		public function storeZip($items) {
+			$id = uniqid();
+			$zip = $this->createZip($items);
+			$this->env->session()->param("zip_".$id, $zip->filename());
+			return $id;
+		}
+		
+		public function downloadStoredZip($id, $mobile) {
+			$p = "zip_".$id;
+			if (!$this->env->session()->hasParam($p))
+				throw new ServiceException("INVALID_REQUEST", "Stored zip id does not exist: ".$p);
+			
+			$filename = $this->env->session()->param($p);
+			if (!file_exists($filename))
+				throw new ServiceException("INVALID_REQUEST", "Stored zip file does not exist");
+
+			$handle = @fopen($filename, "rb");
+			if (!$handle)
+				throw new ServiceException("REQUEST_FAILED", "Could not open zip for reading: ".$this->name);
+			$this->env->response()->download("items.zip", "zip", $mobile, $handle);
+		}
+		
+		private function createZip($items) {
 			$this->env->features()->assertFeature("zip_download");
 			
 			if (is_array($items)) {
 				$this->assertRights($items, Authentication::RIGHTS_READ, "download as zip");
 				
-				$zip = $this->zipper("items.zip");
+				$zip = $this->zipper();
 				foreach($items as $item) {
 					$item->addToZip($zip);
 					$this->env->events()->onEvent(FileEvent::download($item));
@@ -503,22 +610,77 @@
 			} else {
 				$item = $items;
 				$this->assertRights($item, Authentication::RIGHTS_READ, "download as zip");
-				
-				$zip = $this->zipper($item->name().".zip");
+
+				$zip = $this->zipper();
 				$item->addToZip($zip);
 				$zip->finish();
 				
 				$this->env->events()->onEvent(FileEvent::download($item));
 			}
+			
+			return $zip;
 		}
 		
-		public function zipper($name) {
-			require_once('MollifyZipStream.class.php');
-			return new MollifyZipStream($this->env, $name, $this->setting("zip_options"));
+		public function search($parent, $text) {
+			if ($parent == NULL) {
+				$m = array();
+				foreach($this->getRootFolders() as $id => $root) {
+					$data = array();
+					foreach($this->searchers as $searcher)
+						$data[$searcher->key()] = $searcher->preData($root, $text);
+					$m = array_merge($m, $this->searchRecursive($data, $root, $text));
+				}
+			} else {
+				$this->itemIdProvider()->load($parent, TRUE);
+				$data = array();
+				foreach($this->searchers as $searcher)
+					$data[$searcher->key()] = $searcher->preData($parent, $text);
+				$m = $this->searchRecursive($data, $parent, $text);
+			}
+			return array("count" => count($m), "matches" => $m);
 		}
 		
-		public function setting($setting) {
-			return $this->env->settings()->setting($setting);
+		private function searchRecursive($data, $parent, $text) {
+			$result = array();
+			
+			foreach($parent->items() as $item) {
+				$id = $item->id();
+				
+				foreach($this->searchers as $searcher) {
+					$match = $searcher->match($data[$searcher->key()], $item, $text);
+					if (!$match) continue;
+					
+					if (in_array($id, $result)) {
+						$result[$id]["matches"] = array_merge($match, $result[$id]["matches"]);
+					} else {
+						$result[$id] = array("item" => $item->data(), "matches" => $match);
+					}
+				}
+				if (!$item->isFile()) $result = array_merge($result, $this->searchRecursive($data, $item, $text));
+			}
+			return $result;
+		}
+		
+		public function zipper() {
+			require_once('zip/MollifyZip.class.php');
+			$zipper = $this->setting("zipper", TRUE);
+			
+			if (strcasecmp($zipper, "ziparchive") === 0) {
+				require_once('zip/MollifyZipArchive.class.php');
+				return new MollifyZipArchive($this->env);
+			} else if (strcasecmp($zipper, "native") === 0) {
+				require_once('zip/MollifyZipNative.class.php');
+				return new MollifyZipNative($this->env);
+			} else if (strcasecmp($zipper, "raw") === 0) {
+				require_once('zip/MollifyZipRaw.class.php');
+				return new MollifyZipRaw($this->env);
+			}
+			
+			throw new ServiceException("INVALID_CONFIGURATION", "Unsupported zipper configured: ".$zipper);
+		}
+		
+		public function setting($setting, $allowDefaultIfNotDefined = FALSE) {
+			return $this->env->settings()->setting($setting, $allowDefaultIfNotDefined);
 		}
 
 		public function log() {
@@ -528,7 +690,6 @@
 		public function __toString() {
 			return "FILESYSTEMCONTROLLER";
 		}
-
 	 }
 	 
 	 class FileEvent extends Event {
@@ -539,6 +700,7 @@
 		const CREATE_FOLDER = "create_folder";
 		const DOWNLOAD = "download";
 		const UPLOAD = "upload";
+		const VIEW = "view";
 		
 		private $item;
 		private $info;
@@ -550,11 +712,12 @@
 			$eventHandler->registerEventType(FilesystemController::EVENT_TYPE_FILE, self::DELETE, "Delete file");
 			$eventHandler->registerEventType(FilesystemController::EVENT_TYPE_FILE, self::CREATE_FOLDER, "Create folder");
 			$eventHandler->registerEventType(FilesystemController::EVENT_TYPE_FILE, self::DOWNLOAD, "Download file");
+			$eventHandler->registerEventType(FilesystemController::EVENT_TYPE_FILE, self::VIEW, "View file");
 			$eventHandler->registerEventType(FilesystemController::EVENT_TYPE_FILE, self::UPLOAD, "Upload file");
 		}
 		
-		static function rename($item, $name) {
-			return new FileEvent($item, self::RENAME, $name);
+		static function rename($item, $to) {
+			return new FileEvent($item, self::RENAME, $to);
 		}
 
 		static function copy($item, $to) {
@@ -581,6 +744,10 @@
 			return new FileEvent($item, self::UPLOAD);
 		}
 		
+		static function view($item) {
+			return new FileEvent($item, self::VIEW);
+		}
+		
 		function __construct($item, $type, $info = NULL) {
 			parent::__construct(time(), FileSystemController::EVENT_TYPE_FILE, $type);
 			$this->item = $item;
@@ -591,6 +758,10 @@
 			return $this->item;
 		}
 
+		public function info() {
+			return $this->info;
+		}
+		
 		public function itemToStr() {
 			return $this->item->internalPath();
 		}
@@ -598,17 +769,32 @@
 		public function details() {
 			$f = $this->item->id()." (".$this->item->filesystem()->name().")";
 			
-			if ($this->subType() === self::RENAME)
-				return 'item id='.$f.';to='.$this->info;
-			if ($this->subType() === self::COPY or $this->subType() === self::MOVE)
+			if ($this->subType() === self::RENAME or $this->subType() === self::COPY or $this->subType() === self::MOVE)
 				return 'item id='.$f.';to='.$this->info->id()." (".$this->info->filesystem()->name().")";
 			return 'item id='.$f;
 		}
 		
+		public function values($formatter) {
+			$values = parent::values($formatter);
+			$values["item_id"] = $this->item->id();
+			$values["item_name"] = $this->item->name();
+			$values["item_path"] = $this->item->path();
+			$values["item_internal_path"] = $this->item->internalPath();
+			$values["root_name"] = $this->item->root()->name();
+
+			if ($this->subType() === self::RENAME or $this->subType() === self::COPY or $this->subType() === self::MOVE) {
+				$values["to_item_id"] = $this->info->id();
+				$values["to_item_name"] = $this->info->name();
+				$values["to_item_path"] = $this->info->path();
+				$values["to_item_internal_path"] = $this->info->internalPath();
+				$values["to_root_name"] = $this->info->root()->name();
+			}
+
+			return $values;
+		}
+		
 		public function __toString() {
 			return "FILESYSTEMEVENT ".get_class($this);
 		}
-
 	}
-
-?>
\ No newline at end of file
+?>
