@@ -16,16 +16,24 @@
 		private $pw;
 		private $database;
 		private $tablePrefix;
+		private $port;
+		private $socket;
 		
 		private $db = NULL;
 		
-		public function __construct($host, $user, $pw, $database, $tablePrefix) {
-			Logging::logDebug("MySQL DB: ".$user."@".$host.":".$database."(".$tablePrefix.")");
+		public function __construct($host, $user, $pw, $database, $tablePrefix, $port, $socket) {
+			Logging::logDebug("MySQL DB: ".$user."@".$host.":".$database."(".$tablePrefix.") port=".$port." socket=".$socket);
 			$this->host = $host;
 			$this->user = $user;
 			$this->pw = $pw;
 			$this->database = $database;
 			$this->tablePrefix = $tablePrefix;
+			$this->port = $port;
+			$this->socket = $socket;
+		}
+		
+		public function type() {
+			return "mysql";
 		}
 		
 		public function host() {
@@ -52,8 +60,20 @@
 			return $this->db != NULL;
 		}
 		
+		public function port() {
+			return $this->port;
+		}
+		
+		public function socket() {
+			return $this->socket;
+		}
+		
 		public function connect($selectDb = TRUE) {
-			$db = @mysql_connect($this->host, $this->user, $this->pw);
+			$host = $this->host;
+			if ($this->socket != NULL) $host = 'localhost:'.$this->socket;
+			else if ($this->port != NULL) $host .= ":".$this->port;
+			
+			$db = @mysql_connect($host, $this->user, $this->pw);
 			if (!$db) throw new ServiceException("INVALID_CONFIGURATION", "Could not connect to database (host=".$this->host.", user=".$this->user.", password=".$this->pw."), error: ".mysql_error());
 
 			$this->db = $db;			
@@ -105,7 +125,7 @@
 		}
 		
 		public function string($s, $quote = FALSE) {
-			if ($s == NULL) return 'NULL';
+			if ($s === NULL) return 'NULL';
 			$r = mysql_real_escape_string($s, $this->db);
 			if ($quote) return "'".$r."'";
 			return $r;
@@ -116,7 +136,7 @@
 			$first = TRUE;
 			foreach($a as $s) {
 				if (!$first) $result .= ',';
-				if ($quote) $result .= "'".$s-"'";
+				if ($quote) $result .= "'".$s."'";
 				else $result .= $s;
 				$first = FALSE;
 			}
@@ -153,7 +173,28 @@
 			mysql_free_result($this->result);
 			return $list;
 		}
-		
+
+		public function values($col) {
+			$list = array();
+			while ($row = mysql_fetch_assoc($this->result)) {
+				$list[] = $row[$col];
+			}
+			mysql_free_result($this->result);
+			return $list;
+		}
+			
+		public function valueMap($keyCol, $valueCol = NULL) {
+			$list = array();
+			while ($row = mysql_fetch_assoc($this->result)) {
+				if ($valueCol == NULL)
+					$list[$row[$keyCol]] = $row;
+				else
+					$list[$row[$keyCol]] = $row[$valueCol];
+			}
+			mysql_free_result($this->result);
+			return $list;
+		}
+			
 		public function firstRow() {
 			$ret = mysql_fetch_assoc($this->result);
 			mysql_free_result($this->result);
