@@ -16,16 +16,26 @@
 		private $pw;
 		private $database;
 		private $tablePrefix;
-		
+		private $port;
+		private $socket;
+		private $engine;
+
 		private $db = NULL;
 		
-		public function __construct($host, $user, $pw, $database, $tablePrefix) {
+		public function __construct($host, $user, $pw, $database, $tablePrefix, $port, $socket, $engine) {
 			Logging::logDebug("MySQL DB: ".$user."@".$host.":".$database."(".$tablePrefix.")");
 			$this->host = $host;
 			$this->user = $user;
 			$this->pw = $pw;
 			$this->database = $database;
 			$this->tablePrefix = $tablePrefix;
+			$this->port = $port;
+			$this->socket = $socket;
+			$this->engine = $engine;
+		}
+		
+		public function type() {
+			return "mysql";
 		}
 		
 		public function host() {
@@ -48,6 +58,14 @@
 			return $this->tablePrefix;
 		}
 		
+		public function port() {
+			return $this->port;
+		}
+		
+		public function socket() {
+			return $this->socket;
+		}
+		
 		public function isConnected() {
 			return $this->db != NULL;
 		}
@@ -55,12 +73,12 @@
 		public function connect($selectDb = TRUE) {
 			mysqli_report(MYSQLI_REPORT_ALL);
 			try {
-				if ($selectDb) $db = @mysqli_connect($this->host, $this->user, $this->pw, $this->database);
-				else $db = @mysqli_connect($this->host, $this->user, $this->pw);
+				if ($selectDb) $db = @mysqli_connect($this->host, $this->user, $this->pw, $this->database, $this->port, $this->socket);
+				else $db = @mysqli_connect($this->host, $this->user, $this->pw, "", $this->port, $this->socket);
 			} catch (mysqli_sql_exception $e) {
-				throw new ServiceException("INVALID_CONFIGURATION", "Could not connect to database (host=".$this->host.", user=".$this->user.", password=".$this->pw."), error: ".mysqli_connect_error());
+				throw new ServiceException("INVALID_CONFIGURATION", "Could not connect to database (host=".$this->host.", user=".$this->user.", password=".$this->pw."), error: ".mysqli_connect_errno()."/".mysqli_connect_error());
 			}
-			if (!$db) throw new ServiceException("INVALID_CONFIGURATION", "Could not connect to database (host=".$this->host.", user=".$this->user.", password=".$this->pw."), error: ".mysqli_connect_error());
+			if (!$db) throw new ServiceException("INVALID_CONFIGURATION", "Could not connect to database (host=".$this->host.", user=".$this->user.", password=".$this->pw."), error: ".mysqli_connect_errno()."/".mysqli_connect_error());
 
 			$this->db = $db;
 		}
@@ -129,6 +147,7 @@
 			if (!$sql) throw new ServiceException("INVALID_REQUEST", "Error reading sql file (".$file.")");
 
 			$sql = str_replace('{TABLE_PREFIX}', (isset($this->tablePrefix) and $this->tablePrefix != '') ? $this->tablePrefix : '', $sql);
+			$sql = str_replace('{ENGINE}', (isset($this->engine) and $this->engine != '') ? $this->engine : '', $sql);
 			$this->queries($sql);
 		}
 		
@@ -167,10 +186,14 @@
 			if (!$result)
 				throw new ServiceException("INVALID_CONFIGURATION", "Error rollbacking transaction: ".mysqli_error($this->db));
 		}
-		
-		public function string($s) {
-			return mysqli_real_escape_string($this->db, $s);
+				
+		public function string($s, $quote = FALSE) {
+			if ($s === NULL) return 'NULL';
+			$r = mysqli_real_escape_string($this->db, $s);
+			if ($quote) return "'".$r."'";
+			return $r;
 		}
+
 	}
 	
 	class MySQLIResult {
@@ -199,6 +222,15 @@
 			return $list;
 		}
 		
+		public function values($col) {
+			$list = array();
+			while ($row = mysqli_fetch_assoc($this->result)) {
+				$list[] = $row[$col];
+			}
+			mysqli_free_result($this->result);
+			return $list;
+		}
+		
 		public function firstRow() {
 			$ret = mysqli_fetch_assoc($this->result);
 			mysqli_free_result($this->result);
