@@ -1,7 +1,7 @@
 <?php
 
 	/**
-	 * Copyright (c) 2008- Samuli Jrvel
+	 * Copyright (c) 2008- Samuli Järvelä
 	 *
 	 * All rights reserved. This program and the accompanying materials
 	 * are made available under the terms of the Eclipse Public License v1.0
@@ -10,34 +10,50 @@
 	 * this entire header must remain intact.
 	 */
 
-	require_once("install/MollifyInstaller.class.php");
+	require_once("install/MollifyInstallProcessor.class.php");
 	require_once("include/ServiceEnvironment.class.php");
 	require_once("include/mysql/DatabaseUtil.class.php");
 	require_once("install/mysql/MySQLInstallUtil.class.php");
 	
-	class MySQLInstaller extends MollifyInstaller {
+	class MySQLInstaller {
+		protected $processor;
 		private $configured;
 		protected $db;
 
-		public function __construct($type, $settingsVar, $pageRoot = "install") {
-			parent::__construct($pageRoot, $type, $settingsVar);
+		public function __construct($settings, $type = "install") {
+			$this->processor = new MollifyInstallProcessor($type, "mysql", $settings);
 			
-			global $DB_HOST, $DB_USER, $DB_PASSWORD, $DB_DATABASE, $DB_TABLE_PREFIX;
+			global $DB_HOST, $DB_USER, $DB_PASSWORD, $DB_DATABASE, $DB_TABLE_PREFIX, $DB_SOCKET, $DB_PORT, $DB_ENGINE;
 			$this->configured = isset($DB_USER, $DB_PASSWORD);
-			$this->db = $this->createDB($DB_HOST, $DB_USER, $DB_PASSWORD, $DB_DATABASE, $DB_TABLE_PREFIX);
+			$this->db = $this->createDB($DB_HOST, $DB_USER, $DB_PASSWORD, $DB_DATABASE, $DB_TABLE_PREFIX, $DB_PORT, $DB_SOCKET, $DB_ENGINE);
 			$this->dbUtil = new DatabaseUtil($this->db);
 		}
 
-		private function createDB($host, $user, $password, $database, $tablePrefix) {
+		private function createDB($host, $user, $password, $database, $tablePrefix, $port, $socket, $engine) {
 			if (!isset($host)) $host = "localhost";
 			if (!isset($database)) $database = "mollify";
 			if (!isset($tablePrefix)) $tablePrefix = "";
+			if (!isset($port)) $port = NULL;
+			if (!isset($socket)) $socket = NULL;
+			else {
+				$host = NULL;
+				$port = NULL;
+			}
+			if (!isset($engine)) $engine = "innodb";
 			
 			require_once("include/mysql/MySQLIDatabase.class.php");
-			return new MySQLIDatabase($host, $user, $password, $database, $tablePrefix);
+			return new MySQLIDatabase($host, $user, $password, $database, $tablePrefix, $port, $socket, $engine);
 		}
 		
-		protected function util() {
+		public function processor() {
+			return $this->processor;
+		}
+		
+		public function onError($e) {
+			$this->processor->onError($e);
+		}
+		
+		public function util() {
 			require_once("install/mysql/MySQLInstallUtil.class.php");
 			return new MySQLInstallUtil($this->db);
 		}
@@ -94,19 +110,43 @@
 		}
 
 		public function currentVersion() {
-			return MySQLConfigurationProvider::VERSION;
+			return MySQLConfiguration::VERSION;
 		}
 		
 		public function db() {
 			return $this->db;
-		}		
+		}
+		
+		public function action() {
+			return $this->processor->action();
+		}
+
+		public function hasError() {
+			return $this->processor->hasError();
+		}
+
+		public function hasErrorDetails() {
+			return $this->processor->hasErrorDetails();
+		}
+		
+		public function error() {
+			return $this->processor->error();
+		}
+
+		public function errorDetails() {
+			return $this->processor->errorDetails();
+		}
+
+		public function data($name = NULL) {
+			return $this->processor->data($name);
+		}
 
 		public function process() {
 			$this->checkSystem();
 			$this->checkInstalled();
 			$this->checkConfiguration();
 
-			$phase = $this->phase();
+			$phase = $this->processor->phase();
 			if ($phase == NULL) $phase = 'db';
 			Logging::logDebug("Installer phase: [".$phase."]");	
 			
@@ -115,35 +155,35 @@
 		
 		private function checkSystem() {
 			if (!function_exists('mysql_connect')) {
-				$this->setError("MySQL not detected", "Mollify cannot be installed to this system when MySQL is not available. Check your system configuration or choose different configuration type.");
-				$this->showPage("install_error");
+				$this->processor->setError("MySQL not detected", "Mollify cannot be installed to this system when MySQL is not available. Check your system configuration or choose different configuration type.");
+				$this->processor->showPage("install_error");
 			}
 		
 			if (!function_exists('mysqli_multi_query')) {
-				$this->setError("MySQL Improved (mysqli) not detected", "Mollify installer cannot continue without <a href='http://www.php.net/manual/en/mysqli.overview.php' target='_blank'>MySQL Improved</a> installed. Either check your configuration to install or enable this, or install Mollify manually (see instructions <a href='http://code.google.com/p/mollify/wiki/ConfigurationMySql' target='_blank'>here</a>).");
-				$this->showPage("install_error");
+				$this->processor->setError("MySQL Improved (mysqli) not detected", "Mollify installer cannot continue without <a href='http://www.php.net/manual/en/mysqli.overview.php' target='_blank'>MySQL Improved</a> installed. Either check your configuration to install or enable this, or install Mollify manually (see instructions <a href='http://code.google.com/p/mollify/wiki/ConfigurationMySql' target='_blank'>here</a>).");
+				$this->processor->showPage("install_error");
 			}
 		}
 		
 		private function checkInstalled() {
 			if (!$this->isInstalled()) return;
 			
-			$this->createEnvironment();
-			if (!$this->authentication()->isAdmin()) die("Mollify Installer requires administrator user");
+			$this->processor->createEnvironment();
+			if (!$this->processor->authentication()->isAdmin()) die("Mollify Installer requires administrator user");
 			
-			$this->showPage("installed");
+			$this->processor->showPage("installed");
 		}
 		
 		private function checkConfiguration() {
 			if (!$this->isConfigured())
-				$this->showPage("configuration");
+				$this->processor->showPage("configuration");
 
 			try {
 				$this->db->connect(FALSE);
 			} catch (ServiceException $e) {
 				if ($e->type() === 'INVALID_CONFIGURATION') {
-					$this->setError("Could not connect to database", '<code>'.$e->details().'</code>');
-					$this->showPage("configuration");
+					$this->processor->setError("Could not connect to database", '<code>'.$e->details().'</code>');
+					$this->processor->showPage("configuration");
 					die();
 				}
 				throw $e;
@@ -151,7 +191,7 @@
 		}
 		
 		private function onPhase($phase) {
-			$this->setPhase($phase);
+			$this->processor->setPhase($phase);
 			
 			switch ($phase) {
 				case 'db':
@@ -161,7 +201,7 @@
 					$this->onPhaseAdmin();
 					break;
 				case 'success':
-					$this->showPage("success");
+					$this->processor->showPage("success");
 					break;
 				default:
 					Logging::logError("Invalid installer phase: ".$phase);
@@ -172,14 +212,14 @@
 		// PHASES
 				
 		private function onPhaseDatabase() {
-			if ($this->action() === 'continue_db') {
-				$this->clearAction();
+			if ($this->processor->action() === 'continue_db') {
+				$this->processor->clearAction();
 				
 				if (!$this->db->databaseExists()) {
 					try {
 						$this->dbUtil->createDatabase();
 					} catch (ServiceException $e) {
-						$this->setError("Unable to create database", '<code>'.$e->details().'</code>');
+						$this->processor->setError("Unable to create database", '<code>'.$e->details().'</code>');
 						$this->onPhase('db');
 					}
 				}
@@ -187,31 +227,31 @@
 				$this->checkDatabasePermissions();
 			}
 			
-			$this->showPage("database");
+			$this->processor->showPage("database");
 		}
 		
 		private function checkDatabasePermissions() {
 			try {
 				$this->util()->checkPermissions();
 			} catch (ServiceException $e) {
-				$this->setError("Insufficient database permissions", '<code>'.$e->details().'</code>');
-				$this->onPhase('db');
+				$this->processor->setError("Insufficient database permissions", '<code>'.$e->details().'</code>');
+				$this->processor->onPhase('db');
 			}
 			$this->onPhase('admin');
 		}
 		
 		private function onPhaseAdmin() {
-			if ($this->action() === 'install')
+			if ($this->processor->action() === 'install')
 				$this->install();
-			$this->showPage("admin");
+			$this->processor->showPage("admin");
 		}
 		
 		private function install() {
 			try {
 				$this->db->selectDb();
 			} catch (ServiceException $e) {
-				$this->setError("Could not select database", '<code>'.$e->details().'</code>');
-				$this->showPage("install_error");
+				$this->processor->setError("Could not select database", '<code>'.$e->details().'</code>');
+				$this->processor->showPage("install_error");
 			}
 			
 			$this->db->startTransaction();
@@ -220,22 +260,22 @@
 				$this->util()->execCreateTables();
 				$this->util()->execInsertParams();
 			} catch (ServiceException $e) {
-				$this->setError("Could not install", '<code>'.$e->details().'</code>');
-				$this->showPage("install_error");
+				$this->processor->setError("Could not install", '<code>'.$e->details().'</code>');
+				$this->processor->showPage("install_error");
 			}
 
 			try {
 				$this->util()->createAdminUser($this->data("name"), $this->data("password"));
 			} catch (ServiceException $e) {
-				$this->setError("Could not create admin user", '<code>'.$e->details().'</code>');
-				$this->showPage("install_error");
+				$this->processor->setError("Could not create admin user", '<code>'.$e->details().'</code>');
+				$this->processor->showPage("install_error");
 			}
 			
 			try {
 				$this->db->commit();
 			} catch (ServiceException $e) {
-				$this->setError("Could install", '<code>'.$e->details().'</code>');
-				$this->showPage("install_error");
+				$this->processor->setError("Could not install", '<code>'.$e->details().'</code>');
+				$this->processor->showPage("install_error");
 			}
 
 			$this->onPhase('success');
