@@ -11,38 +11,54 @@
 	 */
 
 	$MAIN_PAGE = "update";
-	$updater = NULL;
+	$installer = NULL;
 	
-	set_include_path("..");
-	if (!file_exists("../configuration.php")) die();
-	require("configuration.php");
-	global $SETTINGS, $CONFIGURATION_PROVIDER;
+	set_include_path(realpath('../').PATH_SEPARATOR.get_include_path());
+	chdir("..");
 
-	$installer = createUpdater($CONFIGURATION_PROVIDER, $SETTINGS);
+	if (!file_exists("configuration.php")) die();
+	require("configuration.php");
+	global $SETTINGS, $CONFIGURATION_TYPE;
+
+	try {
+		$installer = createUpdater($CONFIGURATION_TYPE, $SETTINGS);
+	} catch (Exception $e) {
+		showError($e);
+		die();
+	} 
 	try {
 		$installer->process();
 	} catch (Exception $e) {
 		$installer->onError($e);
-	}
-
-	function isValidConfigurationType($type) {
-		$TYPES = array("file","mysql");
-		return in_array(strtolower($type), $TYPES);
+		showError($e);
 	}
 		
 	function createUpdater($type, $settings) {
 		if (!isset($type) or !isValidConfigurationType($type)) die();
 		
+		require_once("update/UpdateController.class.php");
 		switch (strtolower($type)) {
-			case 'file':
-				require_once("update/file/FileUpdater.class.php");
-				return new FileUpdater($type, $settings);
 			case 'mysql':
 				require_once("update/mysql/MySQLUpdater.class.php");
-				return new MySQLUpdater($type, $settings);
+				return new UpdateController(new MySQLUpdater($settings));
+			case 'sqlite':
+				require_once("update/sqlite/SQLiteUpdater.class.php");
+				return new UpdateController(new SQLiteUpdater($settings));
 			default:
 				die("Unsupported updater type: ".$type);
-
+		}
+	}
+	
+	function isValidConfigurationType($type) {
+		return in_array(strtolower($type), array("mysql","sqlite"));
+	}
+	
+	function showError($e) {
+		$c = get_class($e);
+		if ($c === "ServiceException") {
+			echo "Mollify error (".$e->type()."): ".$e->details();
+		} else {
+			echo "Unknown error (".$c."): ".$e->getMessage();
 		}
 	}
 ?>
\ No newline at end of file
